import enum
from typing import Optional, Union, List

from pydantic import BaseModel, validator
from ruamel.yaml import CommentedSeq, CommentedMap


class Target(BaseModel):
    """
    Target object used to find a manifest. This should be a unique
    combination able to detect only one manifest
    """
    apiVersion: str
    kind: str
    name: str


class Action(enum.Enum):
    """Patching actions"""
    add = 'add'
    replace = 'replace'
    remove = 'remove'


class Op(BaseModel):
    """
    Operation

    name: name of the operation
    path: path to the resource in the manifest. In case of list
          you can use item names instead of indexes
    value: value to be added or which will replace old value. Not
           required for action = remove
    action: Action
    """
    name: str
    path: str
    value: Optional[Union[str, int, dict, list]]
    action: Action

    @validator('action')
    def check_value(cls, v, values, **kwargs):
        if v == Action.add or v == Action.replace:
            if 'value' not in values or values['value'] is None:
                raise ValueError(f'Value needs to be present for action {v}')

            # For add the value needs to be an object
            if v == Action.add and type(values['value']) not in [list, dict, CommentedMap, CommentedSeq]:
                raise ValueError('For add, the value needs to be a dict or a list')
        return v


class Patch(BaseModel):
    """Patch object"""
    name: str
    target: Target
    ops: List[Op]



import os

from src.ksux.src.read_file import read_yaml, read_json
from src.ksux.tests.config import settings


def test_read_yaml_resources_separated_by_dashes():
    f = read_yaml(os.path.join(settings.PARENT, 'data', 'read_file', 'two_deployments_separated_by_dashes.yaml'))

    assert type(f) == list
    assert len(f) == 2


def test_read_yaml_resources_as_yaml_list():
    f = read_yaml(os.path.join(settings.PARENT, 'data', 'read_file', 'two_deployments_in_valid_yaml_format.yml'))

    assert type(f) == list
    assert len(f) == 2


def test_read_json():
    f = read_json(os.path.join(settings.PARENT, 'data', 'read_file', 'service_in_json_format.json'))

    assert type(f) == list
    assert len(f) == 1



import json
import logging
import sys
from argparse import ArgumentParser

from ruamel.yaml import round_trip_dump

from .src.create_manifests import patch_manifests
from .src.save_manifests import save_manifests


def main():
    parser = ArgumentParser()

    parser.add_argument('-b', '--base_dir', help='Base directory with yaml/yml/json kubernetes templates')
    parser.add_argument('-o', '--out_dir', help='Output directory', default='out')
    parser.add_argument('-p', '--patches_dir',
                        help='Patches directory. Script will read only files with yaml/yml/json extensions')
    parser.add_argument('-e', '--output_extension', help='Extension of output files', choices=['yaml', 'json', 'yml'],
                        default='json')
    parser.add_argument('--dry-run', help='Print manifests to stdout', action="store_true")
    parser.add_argument('-q', '--quiet', help='Do not print debug, info and warning messages', action='store_true')
    parser.add_argument('--version', help='Package version', action="store_true")

    args = parser.parse_args()

    if args.version:
        print('0.4.1')
        exit(0)

    if args.quiet:
        logging.root.setLevel(logging.ERROR)
    else:
        logging.root.setLevel(logging.INFO)

    manifests = patch_manifests(base_dir=args.base_dir, patches_dir=args.patches_dir)

    if args.dry_run:
        if args.output_extension in ['yaml', 'yml']:
            print('---')
            for i, m in enumerate(manifests):
                round_trip_dump(m, sys.stdout)
                print('---')
        else:
            print(json.dumps(manifests))

    else:
        save_manifests(patched_manifests=manifests, out_dir=args.out_dir, extension=args.output_extension)

    if args.output_extension in ['yaml', 'yml']:
        logging.warning(
            'Make sure to check the manifests, especially if your input was in json format and you had some "add" ops '
            'in your patches. In this case it is impossible to preserve quotes correctly!')
